@@ -4,7 +4,7 @@
 #
 # File: hab4_pki_tree.sh
 #
-# Description: This script generates a basic HAB4 PKI tree for the Freescale
+# Description: This script generates a basic HAB4 PKI tree for the NXP
 #              HAB code signing feature.  What the PKI tree looks like depends
 #              on whether the SRK is chosen to be a CA key or not.  If the
 #              SRKs are chosen to be CA keys then this script will generate the
@@ -39,14 +39,14 @@
 #              including HAB support this option.
 #
 #        (c) Freescale Semiconductor, Inc. 2011, 2013. All rights reserved.
-#        Copyright 2018 NXP
+#        Copyright 2018-2022 NXP
 #
 #
 #-----------------------------------------------------------------------------
 
 printf "\n"
 printf "    +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n"
-printf "    This script is a part of the Code signing tools for Freescale's\n"
+printf "    This script is a part of the Code signing tools for NXP's\n"
 printf "    High Assurance Boot.  It generates a basic PKI tree.  The PKI\n"
 printf "    tree consists of one or more Super Root Keys (SRK), with each\n"
 printf "    SRK having two subordinate keys: \n"
@@ -80,10 +80,11 @@ usage()
     echo "$0"
     echo
     echo "Command Line Mode:"
-    echo "$0 -existing-ca <y/n> [-ca-key <CA key name> -ca-cert <CA cert name>] -use-ecc <y/n> -kl <ECC/RSA Key Length> -duration <years> -num-srk <1-4> -srk-ca <y/n>"
-    echo "Options:"
-    echo "    -kl: -use-ecc = y then Supported key lengths: p256, p384, p521"
-    echo "       : -use-ecc = n then Supported key lengths: 1024, 2048, 3072, 4096"
+    echo "$0 -existing-ca <y/n> [-ca-key <CA key name> -ca-cert <CA cert name>] -kt < rsa/rsa-pss/ecc> -kl <ECC Curve/RSA Key Length> -duration <years> -num-srk <1-4> -srk-ca <y/n>"
+    echo "  Key Type Options:"
+    echo "    -kl ecc     : then Supported key lengths: p256, p384, p521"
+    echo "    -kl rsa     : then Supported key lengths: 1024, 2048, 3072, 4096"
+    echo "    -kl rsa-pss : then Supported key lengths: 1024, 2048, 3072, 4096"
     echo
     echo "$0 -h | --help : This text"
     echo
@@ -138,9 +139,9 @@ then
                 ca_cert=$1
                 shift
                 ;;
-            -use-ecc)
+            -kt)
                 shift
-		        use_ecc=$1
+                kt=$1
                 shift
                 ;;
             -kl)
@@ -189,9 +190,26 @@ fi
 
 if [ "$interactive" = "y" ]
 then
-    printf "Do you want to use Elliptic Curve Cryptography (y/n)?:  \b"
-    read use_ecc
+    printf "\n"
+    printf "Key type options (confirm targeted device supports desired key type):\n"
+    printf "Select the key type (possible values: rsa, rsa-pss, ecc)?:  \b"
+    read kt
 fi
+case $kt in
+    rsa)
+        rsa_algo='rsa'
+        use_ecc="n"
+        use_rsa_pss="n" ;;
+    rsa-pss)
+        rsa_algo='rsa-pss'
+        use_ecc="n"
+        use_rsa_pss="y" ;;
+    ecc)
+        use_ecc="y" ;;
+    *)
+        echo Invalid key type selected
+        exit 1 ;;
+esac
 
 if [ $use_ecc = "y" ]
 then
@@ -205,7 +223,7 @@ then
     # Confirm that a valid key length has been entered
     case $kl in
         p256)
-            cn="prime256v1" ;;
+            cn="secp256r1" ;;
         p384)
             cn="secp384r1" ;;
         p521)
@@ -216,6 +234,7 @@ then
         exit 1 ;;
     esac
 else
+    # Configure the RSA/RSA-PSS key length
     if [ "$interactive" = "y" ]
     then
         printf "Enter key length in bits for PKI tree:  \b"
@@ -382,7 +401,7 @@ then
     openssl x509 -inform PEM -outform DER -in ${ca_cert}.pem -out ${ca_cert}.der
 
     # Cleanup
-    \rm temp_ca.pem
+    \rm temp_ca.pem $eck
 fi
 
 
@@ -402,8 +421,9 @@ then
         if [ $use_ecc = 'n' ]
         then
             # Generate SRK key
-            openssl genrsa -des3 -passout file:./key_pass.txt -f4 \
-                           -out ./temp_srk.pem ${kl}
+            openssl genpkey -algorithm ${rsa_algo} -pkeyopt rsa_keygen_bits:${kl} \
+                            -pass file:./key_pass.txt \
+                            -out ./temp_srk.pem
 
             srk_subj_req=/CN=SRK${i}_sha256_${kl}_65537_v3_usr/
             srk_crt=../crts/SRK${i}_sha256_${kl}_65537_v3_usr_crt
@@ -476,8 +496,9 @@ do
     if [ $use_ecc = 'n' ]
         then
             # Generate SRK key
-            openssl genrsa -des3 -passout file:./key_pass.txt -f4 \
-                           -out ./temp_srk.pem ${kl}
+            openssl genpkey -algorithm ${rsa_algo} -pkeyopt rsa_keygen_bits:${kl} \
+                            -pass file:./key_pass.txt \
+                            -out ./temp_srk.pem
 
             srk_subj_req=/CN=SRK${i}_sha256_${kl}_65537_v3_ca/
             srk_crt=../crts/SRK${i}_sha256_${kl}_65537_v3_ca_crt
@@ -542,8 +563,9 @@ do
             srk_crt_i=../crts/SRK${i}_sha256_${kl}_65537_v3_ca_crt.pem
             srk_key_i=./SRK${i}_sha256_${kl}_65537_v3_ca_key.pem
             # Generate key
-            openssl genrsa -des3 -passout file:./key_pass.txt -f4 \
-                           -out ./temp_csf.pem ${kl}
+            openssl genpkey -algorithm ${rsa_algo} -pkeyopt rsa_keygen_bits:${kl} \
+                            -pass file:./key_pass.txt \
+                            -out ./temp_csf.pem
 
             csf_subj_req=/CN=CSF${i}_1_sha256_${kl}_65537_v3_usr/
             csf_crt=../crts/CSF${i}_1_sha256_${kl}_65537_v3_usr_crt
@@ -607,8 +629,9 @@ do
     if [ $use_ecc = 'n' ]
         then
             # Generate key
-            openssl genrsa -des3 -passout file:./key_pass.txt -f4 \
-                           -out ./temp_img.pem ${kl}
+            openssl genpkey -algorithm ${rsa_algo} -pkeyopt rsa_keygen_bits:${kl} \
+                            -pass file:./key_pass.txt \
+                            -out ./temp_img.pem
 
             img_subj_req=/CN=IMG${i}_1_sha256_${kl}_65537_v3_usr/
             img_crt=../crts/IMG${i}_1_sha256_${kl}_65537_v3_usr_crt
